@file:Suppress("MemberVisibilityCanBePrivate")

package no.vegvesen.meldingsformidler.common.domain

import com.fasterxml.jackson.annotation.JsonTypeInfo
import java.time.Instant
import java.time.LocalTime
import java.time.MonthDay
import java.time.Year
@JsonTypeInfo(use = JsonTypeInfo.Id.CLASS, include = JsonTypeInfo.As.PROPERTY, property = "@class")
sealed class Gyldighetsperiode

@Suppress("unused")
data class ArligRepeterendePeriode(
    val validMonths: ValidMonths,
    val validYears: ValidYears? = FOREVER
) : Gyldighetsperiode() {
    companion object {
        val FOREVER : ValidYears? = null
    }
}

data class TimeOfMonth(
    val time: LocalTime = LocalTime.MIDNIGHT,
    val monthDay: MonthDay
) : Comparable<TimeOfMonth> {
    override fun compareTo(other: TimeOfMonth): Int {
        val monthDayComparison = monthDay.compareTo(other.monthDay)
        return when {
            monthDayComparison != 0 -> monthDayComparison
            else -> time.compareTo(other.time)
        }
    }
}

@Suppress("unused")
data class Period(
    val validRange: ValidRange,
) : Gyldighetsperiode()

data class ValidMonths(
    val start: TimeOfMonth,
    val endInclusive: TimeOfMonth
)

data class ValidYears(
    val start: Year?,
    val endInclusive: Year?
)

data class ValidRange(
    val start: Instant,
    val endInclusive: Instant
)
