package no.vegvesen.meldingsformidler.common.filter

import jakarta.servlet.Filter
import jakarta.servlet.FilterChain
import jakarta.servlet.ServletException
import jakarta.servlet.ServletRequest
import jakarta.servlet.ServletResponse
import jakarta.servlet.http.HttpServletRequest
import jakarta.servlet.http.HttpServletRequestWrapper
import org.slf4j.MDC
import java.io.IOException


@Suppress("unused")
class ContentIdFilter : Filter {
    @Throws(IOException::class, ServletException::class)
    override fun doFilter(req: ServletRequest?, resp: ServletResponse?, chain: FilterChain) {
        val contentId = (HttpServletRequestWrapper(req as HttpServletRequest?)).getHeader(CONTENT_ID_HEADER)

        if(contentId != null) {
            MDC.put(CONTENT_ID_HEADER, contentId)
        }

        try {
            chain.doFilter(req, resp)
        } finally {
            MDC.remove(CONTENT_ID_HEADER)
        }
    }

    override fun destroy() {
        MDC.remove(CONTENT_ID_HEADER)
    }

    companion object {
        const val CONTENT_ID_HEADER = "Content-Id"
    }
}

data class ContentId(val value: String)
